@@ -20,9 +20,8 @@ struct MD5Context {
 };
 
 void MD5Init(struct MD5Context *context);
-void MD5Update(struct MD5Context *context, unsigned char const *buf,
-	       unsigned len);
-void MD5Final(unsigned char digest[16], struct MD5Context *context);
+void MD5Update(struct MD5Context *context, const void *buf, unsigned len);
+void MD5Final(void *digest, struct MD5Context *context);
 void MD5Transform(uint32 buf[4], uint32 const in[16]);
 
 char *MD5Digest (unsigned const char *s);
